##' Create a DNA mixture model
##'
##' A model for analysis of one or more DNA mixtures.
##'
##' @details For getting started, see in particular \code{\link{DNAmixtures}}.
##' 
##' The names for known contributors can be chosen freely, whereas
##' unknown contributors are always termed \code{U1,U2, ...}.
##'
##' If amelogenin is included in the analysis, the marker should be
##' named "AMEL" and an integer coding such as X=0, Y=1, where X is
##' assigned a lower number than Y, should be used. Note that in terms
##' of amelogenin, the allele frequencies have a slighly different
##' interpretation to that at other markers, in that they denote the
##' probability of having an \emph{additional} X or Y to the X that
##' all people have. Thus, a natural choice will be p(X)=p(Y)=0.5,
##' denoting equal probability of a male or female contributor.
##' 
##' @param data A list of data.frames, one for each trace. Note, that in
##' the special case of analysing just one trace, this still has to be
##' specified as list(data). Each dataset should contain variables
##' \code{marker}, \code{allele}, and \code{frequency}. Optionally also a
##' column for each reference profile specified in \code{K}.
##' @param k Number of contributors.
##' @param C A vector or list of thresholds for each dataset (trace). 
##' @param K Names of reference profiles.
##' @param database A data.frame containing at least variables "marker", "allele", "frequency".
##' @param reference.profiles A data.frame containing allele counts for each reference profile, if not specified in \code{data}.
##' @param dir Location of network files.
##' @param domainnamelist Names of marker-wise network files (without hkb-extension). Default is the set of markers.
##' @param load Read networks from disk?
##' @param write Save networks as hkb files?
##' @param dyes A list containing a list of dyes indexed by markers
##' @param triangulate Triangulate the networks? Default is to
##' triangulate the network using a good elimination order.
##' @param compile Compile the networks?
##' @param compress Compress the network? Defaults to TRUE and is
##' strongly recommended for models with a large number of
##' contributors.
##' @param use.order Should the default elimination order be used for triangulation?
##' Otherwise the "total.weight" heuristic for triangulation in HUGIN is used.
##' @return An object of class DNAmixture
##' @export
##' @author Therese Graversen
##' @examples
##' data(MC15, MC18, USCaucasian)
##' DNAmixture(list(MC15, MC18), C = list(50,50), k = 3, K = c("K1", "K2"), database = USCaucasian)
##' DNAmixture(list(MC15, MC18), C = list(50,50), k = 3, K = c("K3", "K1", "K2"), database = USCaucasian)
DNAmixture <- function(data, k, C, database, K = character(0),
                       reference.profiles = NULL,
                       dir = character(0), domainnamelist = NULL,
                       load = FALSE, write = FALSE,
                       dyes = NULL, triangulate = TRUE, compile = TRUE, compress = TRUE, use.order = TRUE){

  ## Names of the datasets used, to be stores as the "call"
  call <- deparse(substitute(data))

  n.unknown <- k - length(K)
  if (n.unknown < 0) stop("Too many known contributors specified")
  if (length(C) != length(data)) stop("There needs to be one threshold per trace")
  if (!is.null(dyes) & length(dyes) != length(data)) stop("Specify a dye per trace")
  stopifnot(class(data)=="list")
    
  ntraces <- length(data)
  
  ## Any peaks being censored by the chosen threshold?
  for (r in 1:ntraces){
    if (any((data[[r]]$height < C[[r]]) & (data[[r]]$height > 0)))
      warning("Some observed peaks are below the given detection threshold and are set to 0",
              call. = FALSE, immediate. = TRUE)
    data[[r]]$height <- ifelse(data[[r]]$height < C[[r]], 0, data[[r]]$height)
  }

  data <- DNAmixtureData(data, database = database, reference.profiles = reference.profiles, K = K)
  observed <- lapply(data, function(d)as.vector(which(apply(d[,1:ntraces + 1, drop=F], 2, function(x)any(!is.na(x))))))
#  lapply(data, function(d)as.vector(which(sapply(d[,1:ntraces + 1], function(x)any(!is.na(x))))))

  out <- list(
           ## Markers
           markers = names(data),
           
           ## Data
           data = data,

           ## Number of traces
           ntraces = ntraces,
           
           ## Number of contributors
           k = k,
           
           ## Number of unknown contributors
           n.unknown = n.unknown,
           
           ## Names of known contributors
           K = K,

           ## Detection threshold. 
           C = C,
           
           ## Parameters
           parameters = NULL,
           
           ## Scaling factors for CPTs on O-nodes, if relevant
           CPTfactors = NULL,

           ## Which traces are observed for given markers
           observed = observed,

           ## Dyes used in the kit
           dyes = dyes,

           call = call
           )
  
  ## If there are unknown contributors we need networks
  if (n.unknown > 0){
    
    ## Names for any unknown contributors. Used in labeling parameters
    out$U <- paste("U", seq_len(n.unknown), sep = "")

    ## Default directory is of the form xU/ for x unknown contributors
    if (length(dir) == 0){
      dir <- paste(n.unknown, "U/", sep = "")
    }

    ## Default domain names are the marker names
    if (is.null(domainnamelist)){
      domainnamelist <- as.list(names(data))
      names(domainnamelist) <- names(data)
    }
    
    if (!load){      
      ## Build RHugin domains
      res <- buildMixtureDomains(n.unknown, data, domainnamelist = domainnamelist,
                                 write = write, dir = dir, ntraces = ntraces,
                                 triangulate = triangulate, compile = compile, compress = compress, use.order = use.order)
      
      ## store the compression percentages
      out$compressionpct <- res$compressionpct
      
      ## list of pointers to domains    
      out$domains <- res$domains
    }
    else {      
      ## read the rhd-files and store the list of pointers
      out$domains <- lapply(domainnamelist, function(d){read.rhd(paste(dir, d, ".hkb", sep = ""))})
    }

    ## Add the names for binary nodes as attributes to each domain
    out$domains <- lapply(out$markers, function(m){
      als <- seq_len(nrow(out$data[[m]])) ## 1:(number of alleles)
      height <- out$heightnames
      O <- D <- Q <- list()
      for (r in 1:ntraces){
        if (r %in% observed){
          O[[r]] <- paste("O",r, als, sep = "_")
          D[[r]] <- paste("D",r, als, sep = "_")
          Q[[r]] <- paste("Q",r, als, sep = "_")
        } else {
          O[[r]] <- D[[r]] <- Q[[r]] <- character(0)
        }
      }
      O <- lapply(1:ntraces, function(r)if(all(is.na(out$data[[m]][,r+1])))NULL else paste("O",r, als, sep = "_"))
      D <- lapply(1:ntraces, function(r)if(all(is.na(out$data[[m]][,r+1])))NULL else paste("D",r, als, sep = "_"))
      Q <- lapply(1:ntraces, function(r)if(all(is.na(out$data[[m]][,r+1])))NULL else paste("Q",r, als, sep = "_"))
      
      structure(out$domains[[m]],
                O = O,
                D = D,
                Q = Q,
                n = outer(als, seq_len(n.unknown), 
                  function(a, i)paste("n", i, a, sep = "_")))
    })
    names(out$domains) <- out$markers
  }
  
  else {
    ## no unknown contributors
    out$U <- character(0)
  }
  
  class(out) <- "DNAmixture"
  out  
}



##' Print a DNA mixture
##'
##' @export
##' @method print DNAmixture
##' @S3method print DNAmixture
##' @param x A DNA mixture
##' @param ... (not used)
##' @seealso \code{\link{DNAmixture}}
print.DNAmixture <- function(x, ...){

  ## Printing the model:
  cat("\nA DNA mixture model with", x$k, "contributors.\n\n")
  if (length(x$K)){
    cat("Known:", x$K, "\n")
  }
  else cat("No known contributors\n")

  if (x$n.unknown > 0){
    cat("Unknown:", x$U, "\n\n")
  }
  else cat("No unknown contributors\n\n")

  cat("Traces included:", x$call, "\n")
  cat("Detection threshold(s):", unlist(x$C), "\n")

  if (!is.null(x$parameters)){
    cat("Parameters:\n")
    print(parameters(x))
  }
  cat("\n")
  invisible(x)
}
