\docType{package}
\name{DNAmixtures-package}
\alias{DNAmixtures}
\alias{DNAmixtures-package}
\title{Statistical Inference for Mixed Traces of DNA}
\description{
  Tools for statistical inference about contributors to one
  or more mixed traces of DNA.
}
\details{
  The package implements a statistical model for analysis
  of mixed traces of DNA with artefacts such as dropout and
  stutter. In theory, the implementation allows analysis
  with an arbitrary number of unknown contributors.
  However, in practice, depending on hardware and
  time-constraints working with up to 5 or 6 unknown
  contributors seems realistic.
}
\section{Summary of the model implemented}{
  Genotypes for unknown contributors are modelled using
  allele-frequencies from a database specified by the user.

  The peak heights are modelled using gamma distributions
  with the peak for allele a having shape parameter
  \deqn{\rho(1-\xi_a)\sum_a n_{ia}\phi_i + \rho
  \xi_{a+1}\sum_a n_{ia+1}\phi_i} where \eqn{n_{ia}}
  denotes the number of alleles of type a for contributor
  i. Peaks below the detection threshold C are recorded
  with height 0.

  The model parameters are for each DNA mixture \describe{
  \item{phi}{The proportions of DNA from each contributor.}
  \item{rho}{Amplification parameter, which will be larger
  for larger amounts of DNA amplified.} \item{eta}{Scale
  parameter for the gamma distribution.} \item{xi}{Stutter
  parameter. Allele a has stutter \eqn{\xi_a = \xi} or
  \eqn{\xi_a = 0}, depending on whether or not the allele
  can stutter.} }

  Note that the parameter \eqn{\theta} has been renamed to
  \eqn{\phi} to follow the paper describing the implemented
  statistical model (http://arxiv.org/abs/1302.4404). This
  was originally done to avoid conflict of notation with
  the \eqn{\theta}-correction. A change of notation in
  present package is likely in the future.

  The model assumes the model parameters to be the same
  across markers.  Relaxations of these assumptions are
  possible, but are not implemented here.

  An alternative parametrisation uses \eqn{\mu = \rho \eta}
  and \eqn{\sigma = 1/\sqrt{\rho}}, which can be
  interpreted as the mean peak height and the coefficient
  of variation respectively. Another advantage of this
  reparametrisation is that the parameters are fairly
  orthogonal.

  Importantly, note that \code{DNAmixture} when given a
  list of traces will assign the same set of contributors
  to all mixed profiles analysed, but also that this
  actually covers all possible models. This is because a
  contribution-proportion of 0 corresponds to "not having
  contributed".
}

\section{Computation by auxiliary variables}{
  The workhorses of this package are the functions
  \code{\link{set.CPT.O}}, \code{\link{set.CPT.D}} and
  \code{\link{set.CPT.Q}} for setting the conditional
  probability tables on binary nodes. For details, see the
  forthcoming DPhil thesis of Therese Graversen.

  Note that if invalid tables are set -- for instance if
  very extreme parameter values are used -- then any
  subsequent propagation will fail. No roll-back
  functionality has so far been implemented to fix this,
  and the easiest solution is to re-fit the mixture model.
}

\section{Amelogenin}{
  As an experiment, it is possible to add the marker
  Amelogenin, provided that the marker is named as "AMEL"
  and that the coding of alleles X and Y is of a particular
  form. One example of a suitable form is the coding X = 0
  and Y = 1. The allele frequencies used should then also
  contain a marker "AMEL", and here frequencies have a
  slightly different interpretation than for the rest of
  the markers: As all people possess one X, the frequencies
  of X and Y denote the presence of an additional X or Y
  respectively, and thus the frequencies correspond
  directly to the proportions of the two genders.
}
\examples{
## Analysing trace MC18, using USCaucasian allele-frequencies.
## For layout as an EPG we follow the layout of SGMplus.
data(MC18, USCaucasian, SGMplusDyes)

## The prosecution hypothesis could be that K1, K2, and K3 have
## contributed to the trace. Setting detection threshold to 50rfu.
mixHp <- DNAmixture(list(MC18), k = 3, K = c("K1", "K2", "K3"), C = list(50),
                  database = USCaucasian, dyes = list(SGMplusDyes))
plot(mixHp, plot.type = "epg")

## Fit by ML using p as the initial value for optimisation
p <- mixpar(parlist = list(list(rho = 30, eta = 34, xi = 0.08,
              phi = c(K1 = 0.71, K3 = 0.1, K2 = 0.19))))
mlHp <- mixML(mixHp, pars = p)

## Find the estimated covariance matrix of the MLE
V.Hp <- varEst(mixHp, mlHp$mle, npars = list(rho=1,eta=1,xi=1,phi=1))
V.Hp$cov ## using (rho, eta)

## Assess the fit of the distribution of peak heights
qqpeak(mixHp, pars = mlHp$mle, dist = "conditional")

## Assess the prediction of allele presence
pr <- prequential.score(mixHp, pars = mlHp$mle)
plot(pr)

## Compare observed peak heights to peak heights simulated under the model
sims <- rPeakHeight(mixHp, mlHp$mle, nsim = 100, dist = "conditional")
par(mfrow = c(2,5), mar = c(2,2,2,0))
boxplot(mixHp, sims)

## The following part requires a full installation of Hugin, due to
## the size of the networks needed when including unknown
## contributors.

## A defence hypothesis could be that one unknown person has
## contributed along with K1 and K3.
mixHd <- DNAmixture(list(MC18), k = 3, K = c("K1", "K3"), C = list(50),
                  database = USCaucasian, dyes = list(SGMplusDyes))

## Fit by ML -- this takes a bit of time.
mlHd <- mixML(mixHd, pars = mixpar(parlist = list(list(rho = 30, eta = 34, xi = 0.08,
                                    phi = c(K1 = 0.71, K3 = 0.1, U1 = 0.19)))))

## Find the estimated covariance matrix of the MLE
V <- varEst(mixHd, mlHd$mle, npars = list(rho=1,eta=1,xi=1,phi=1))
V$cov ## using (rho, eta)

## Assess the peak height distribution
qq <- qqpeak(mixHd, pars = mlHd$mle, dist = "conditional")

## Assess the prediction of allele presence
pr <- prequential.score(mixHd, pars = mlHd$mle)
plot(pr, col = pr$trace)

## Compare observed peak heights to peak heights simulated under the model
sims <- rPeakHeight(mixHd, mlHd$mle, nsim = 100, dist = "conditional")
par(mfrow = c(2,5), mar = c(2,2,2,0))
boxplot(mixHd, sims)


## The log10 of likelihood-ratio can be found as
(mlHp$lik - mlHd$lik)/log(10)
}
\author{
  Therese Graversen

  Maintainer: Therese Graversen
  \email{therese.graversen@stats.ox.ac.uk}
}
\references{
  Details on the implemented model will be available in the
  forthcoming Ph.d. thesis by Therese Graversen and can
  also be found at \url{http://arxiv.org/abs/1302.4404}.
}
\seealso{
  \code{\link[RHugin]{RHugin}}
}

